@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkConsole.c,v 1.6 1999/04/28 18:18:06 redman Exp $
+ * RCS: @(#) $Id: tkConsole.c,v 1.11.2.1 2001/04/04 07:57:16 hobbs Exp $
  */
 
 #include "tk.h"
@@ -85,6 +85,123 @@ static Tcl_ChannelType consoleChannelType = {
     ConsoleWatch,		/* Watch for events on console. */
     ConsoleHandle,		/* Get a handle from the device. */
 };
+
+
+#ifdef __WIN32__
+
+#include <windows.h>
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * ShouldUseConsoleChannel
+ *
+ * 	Check to see if console window should be used for a given
+ *      standard channel
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Creates the console channel and installs it as the standard
+ *	channels.
+ *
+ *----------------------------------------------------------------------
+ */
+static int ShouldUseConsoleChannel(type)
+    int type;
+{
+    DWORD handleId;		/* Standard handle to retrieve. */
+    DCB dcb;
+    DWORD consoleParams;
+    DWORD fileType;
+    int mode;
+    char *bufMode;
+    HANDLE handle;
+
+    switch (type) {
+	case TCL_STDIN:
+	    handleId = STD_INPUT_HANDLE;
+	    mode = TCL_READABLE;
+	    bufMode = "line";
+	    break;
+	case TCL_STDOUT:
+	    handleId = STD_OUTPUT_HANDLE;
+	    mode = TCL_WRITABLE;
+	    bufMode = "line";
+	    break;
+	case TCL_STDERR:
+	    handleId = STD_ERROR_HANDLE;
+	    mode = TCL_WRITABLE;
+	    bufMode = "none";
+	    break;
+	default:
+	    return 0;
+	    break;
+    }
+
+    handle = GetStdHandle(handleId);
+
+    /*
+     * Note that we need to check for 0 because Windows will return 0 if this
+     * is not a console mode application, even though this is not a valid
+     * handle. 
+     */
+
+    if ((handle == INVALID_HANDLE_VALUE) || (handle == 0)) {
+	return 1;
+    }
+
+    /*
+     * Win2K BUG: GetStdHandle(STD_OUTPUT_HANDLE) can return what appears
+     * to be a valid handle.  See TclpGetDefaultStdChannel() for this change
+     * implemented.  We didn't change it here because GetFileType() [below]
+     * will catch this with FILE_TYPE_UNKNOWN and appropriately return a
+     * value of 1, anyways.
+     *
+     *    char dummyBuff[1];
+     *    DWORD dummyWritten;
+     *
+     *    if ((type == TCL_STDOUT)
+     *	    && !WriteFile(handle, dummyBuff, 0, &dummyWritten, NULL)) {
+     *	return 1;
+     *    }
+     */
+
+    fileType = GetFileType(handle);
+
+    /*
+     * If the file is a character device, we need to try to figure out
+     * whether it is a serial port, a console, or something else.  We
+     * test for the console case first because this is more common.
+     */
+
+    if (fileType == FILE_TYPE_CHAR) {
+	dcb.DCBlength = sizeof( DCB ) ;
+	if (!GetConsoleMode(handle, &consoleParams) &&
+		!GetCommState(handle, &dcb)) {
+	    /*
+	     * Don't use a CHAR type channel for stdio, otherwise Tk
+	     * runs into trouble with the MS DevStudio debugger.
+	     */
+	    
+	    return 1;
+	}
+    } else if (fileType == FILE_TYPE_UNKNOWN) {
+	return 1;
+    } else if (Tcl_GetStdChannel(type) == NULL) {
+	return 1;
+    }
+
+    return 0;
+}
+#else
+/*
+ * Mac should always use a console channel, Unix should if it's trying to
+ */
+
+#define ShouldUseConsoleChannel(chan) (1)
+#endif
 
 /*
  *----------------------------------------------------------------------
@@ -127,9 +244,18 @@ Tk_InitConsoleChannels(interp)
 	
 	/*
 	 * check for STDIN, otherwise create it
+	 *
+	 * Don't do this check on the Mac, because it is hard to prevent
+	 * callbacks from the SIOUX layer from opening stdout & stdin, but
+	 * we don't want to use the SIOUX console.  Since the console is not
+	 * actually created till something is written to the channel, it is
+	 * okay to just ignore it here.
+	 *
+	 * This is still a bit of a hack, however, and should be cleaned up
+	 * when we have a better abstraction for the console.
 	 */
 
-	if (Tcl_GetStdChannel(TCL_STDIN) == NULL) {
+	if (ShouldUseConsoleChannel(TCL_STDIN)) {
 	    consoleChannel = Tcl_CreateChannel(&consoleChannelType, "console0",
 		    (ClientData) TCL_STDIN, TCL_READABLE);
 	    if (consoleChannel != NULL) {
@@ -147,7 +273,7 @@ Tk_InitConsoleChannels(interp)
 	 * check for STDOUT, otherwise create it
 	 */
 	
-	if (Tcl_GetStdChannel(TCL_STDOUT) == NULL) {
+	if (ShouldUseConsoleChannel(TCL_STDOUT)) {
 	    consoleChannel = Tcl_CreateChannel(&consoleChannelType, "console1",
 		    (ClientData) TCL_STDOUT, TCL_WRITABLE);
 	    if (consoleChannel != NULL) {
@@ -165,7 +291,7 @@ Tk_InitConsoleChannels(interp)
 	 * check for STDERR, otherwise create it
 	 */
 	
-	if (Tcl_GetStdChannel(TCL_STDERR) == NULL) {
+	if (ShouldUseConsoleChannel(TCL_STDERR)) {
 	    consoleChannel = Tcl_CreateChannel(&consoleChannelType, "console2",
 		    (ClientData) TCL_STDERR, TCL_WRITABLE);
 	    if (consoleChannel != NULL) {
@@ -438,7 +564,7 @@ ConsoleCmd(clientData, interp, argc, argv)
 {
     ConsoleInfo *info = (ConsoleInfo *) clientData;
     char c;
-    int length;
+    size_t length;
     int result;
     Tcl_Interp *consoleInterp;
     Tcl_DString dString;
@@ -515,7 +641,7 @@ InterpreterCmd(clientData, interp, argc, argv)
 {
     ConsoleInfo *info = (ConsoleInfo *) clientData;
     char c;
-    int length;
+    size_t length;
     int result;
     Tcl_Interp *otherInterp;
 
@@ -564,7 +690,7 @@ InterpreterCmd(clientData, interp, argc, argv)
  *----------------------------------------------------------------------
  */
 
-void 
+static void
 ConsoleDeleteProc(clientData) 
     ClientData clientData;
 {
@@ -679,7 +805,7 @@ TkConsolePrint(interp, devId, buffer, size)
     Tcl_DStringAppend(&output, buffer, size);
 
     Tcl_DStringInit(&command);
-    Tcl_DStringAppend(&command, cmd, strlen(cmd));
+    Tcl_DStringAppend(&command, cmd, (int) strlen(cmd));
     Tcl_DStringAppendElement(&command, output.string);
 
     consoleInterp = info->consoleInterp;
