@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacAppInit.c,v 1.8 1999/04/16 01:51:29 stanton Exp $
+ * RCS: @(#) $Id: tkMacAppInit.c,v 1.12 2000/02/10 08:53:06 jingham Exp $
  */
 
 #include <Gestalt.h>
@@ -23,6 +23,7 @@
 #include "tk.h"
 #include "tkInt.h"
 #include "tkMacInt.h"
+#include "tclInt.h"
 #include "tclMac.h"
 
 #ifdef TK_TEST
@@ -54,8 +55,6 @@ short			SIOUXHandleOneEvent _ANSI_ARGS_((EventRecord *event));
  * Prototypes for functions from the tkConsole.c file.
  */
  
-EXTERN void		TkConsoleCreate _ANSI_ARGS_((void));
-EXTERN int		TkConsoleInit _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN void		TkConsolePrint _ANSI_ARGS_((Tcl_Interp *interp,
 			    int devId, char *buffer, long size));
 /*
@@ -97,6 +96,15 @@ main(
     argc = 1;
     newArgv[0] = "Wish";
     newArgv[1] = NULL;
+    
+    /* Tk_Main is actually #defined to 
+     *     Tk_MainEx(argc, argv, Tcl_AppInit, Tcl_CreateInterp())
+     * Unfortunately, you also HAVE to call Tcl_FindExecutable
+     * BEFORE creating the first interp, or the tcl_library will not
+     * get set properly.  So we call it by hand here...
+     */
+    
+    Tcl_FindExecutable(newArgv[0]);
     Tk_Main(argc, newArgv, Tcl_AppInit);
 }
 
@@ -242,7 +250,11 @@ MacintoshInit()
 
     InitGraf(&tcl_macQdPtr->thePort);
     InitFonts();
+    if (TkMacHaveAppearance() >= 0x110) {
+        InitFloatingWindows();
+    } else {
     InitWindows();
+    }
     InitMenus();
     InitDialogs((long) NULL);		
     InitCursor();
@@ -274,7 +286,6 @@ MacintoshInit()
 
 
     Tcl_MacSetEventProc(TkMacConvertEvent);
-
     return TCL_OK;
 }
 
@@ -311,7 +322,7 @@ SetupMainInterp(
 
     if (strcmp(Tcl_GetVar(interp, "tcl_interactive", TCL_GLOBAL_ONLY), "1")
 	    == 0) {
-	if (TkConsoleInit(interp) == TCL_ERROR) {
+	if (Tk_CreateConsoleWindow(interp) == TCL_ERROR) {
 	    goto error;
 	}
     }
