@@ -8,11 +8,13 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkMacApplication.r 1.3 97/11/03 17:16:24
+ * RCS: @(#) $Id: tkMacApplication.r,v 1.4.2.1 2001/04/04 21:39:48 hobbs Exp $
  */
 
 #include <Types.r>
 #include <SysTypes.r>
+#include <Balloons.r>
+#include <BalloonTypes.r>
 #include <AEUserTermTypes.r>
 
 /*
@@ -43,14 +45,14 @@ resource 'vers' (1) {
 	TK_MAJOR_VERSION, MINOR_VERSION,
 	RELEASE_LEVEL, 0x00, verUS,
 	TK_PATCH_LEVEL,
-	TK_PATCH_LEVEL ", by Ray Johnson  1993-1996" "\n" "Sun Microsystems Labratories"
+	TK_PATCH_LEVEL ", by Ray Johnson & Jim Ingham" "\n" " 2000 Ajuba Solutions"
 };
 
 resource 'vers' (2) {
 	TK_MAJOR_VERSION, MINOR_VERSION,
 	RELEASE_LEVEL, 0x00, verUS,
 	TK_PATCH_LEVEL,
-	"Wish " TK_PATCH_LEVEL "  1993-1996"
+	"Wish " TK_PATCH_LEVEL "  1993-2000"
 };
 
 #define TK_APP_RESOURCES 128
@@ -92,7 +94,7 @@ resource 'FREF' (TK_APP_RESOURCES+1, purgeable)
 
 type TK_APP_CREATOR as 'STR ';
 resource TK_APP_CREATOR (0, purgeable) {
-	"Wish " TK_PATCH_LEVEL "  1996"
+	"Wish " TK_PATCH_LEVEL "  1993-2000"
 };
 
 /*
@@ -110,6 +112,17 @@ resource 'kind' (TK_APP_RESOURCES, "Tcl kind", purgeable) {
 	}
 };
 
+#define kIconHelpString 256
+
+resource 'hfdr' (-5696, purgeable) {
+   HelpMgrVersion, hmDefaultOptions, 0, 0,
+   {HMSTRResItem {kIconHelpString}}
+};
+resource 'STR ' (kIconHelpString, purgeable) {
+   "This is the interpreter for Tcl & Tk scripts"
+   " running on Macintosh computers."
+};
+
 /*
  * The following resource define the icon used by Tcl scripts.  Any
  * TEXT file with the creator of WIsH will get this icon.
